import React, {
  ReactNode,
  useCallback,
  useEffect,
  useReducer,
  useRef,
  useState,
} from "react";
import styles from "./index.module.scss";
import { APIGetChatMsg } from "@/utils/api";
import meAvatar from "../../assets/me.jpg";
import chatgptAvatar from "../../assets/chatgpt_avatar.png";
import LJNEmptyLine from "@/components/LJNEmptyLine";
import dayjs from "dayjs";
import LJNPopup from "@/components/LJNPopup";
import LJNButton from "@/components/LJNButton";

type Props = {};
export default function index({}: Props) {
  // 产生ID
  const generateId = useCallback(() => {
    const unixtime = new Date().getTime();
    return unixtime + "" + Math.trunc(Math.random() * 10000);
  }, []);

  // 消息列表
  const [msgList, dispatch] = useReducer(
    (
      state: Map<any, IMessage | ITimeMessage>,
      action: {
        type: string;
        payload: any;
      }
    ) => {
      const unixtime = new Date().getTime();
      switch (action.type) {
        // 加入消息
        case "addMessage":
          state.set(action.payload.id, {
            id: action.payload.id,
            type: "message",
            message: action.payload.message,
            from_user: action.payload.from_user,
            to_user: action.payload.to_user,
            time: unixtime,
          });
          break;

        // 清空消息
        case "clearMessage":
          localStorage.clear();
          return new Map();
          break;

        // 修改消息
        case "modifyMessage":
          state.set(action.payload.id, {
            id: action.payload.id,
            type: "message",
            message: action.payload.message,
            from_user: action.payload.from_user,
            to_user: action.payload.to_user,
            time: unixtime,
          });
          break;
      }

      return new Map(state);
    },
    new Map(),
    () => {
      let map: Map<any, IMessage | ITimeMessage> = new Map();

      const messages = localStorage.getItem("messages");
      if (messages) {
        map = new Map(JSON.parse(messages));
      }

      return map;
    }
  );

  // 刷新缓存
  useEffect(() => {
    const saveStr = JSON.stringify(Array.from(msgList));
    localStorage.setItem("messages", saveStr);
    if (messageBox.current)
      messageBox.current.scrollTo(0, messageBox.current.scrollHeight);
  }, [msgList]);

  // 输入框信息
  const [message, setMessage] = useState("");

  // 消息盒子
  const messageBox = useRef<HTMLDivElement>(null);

  // 聊天窗口滚动到底部
  const messageBoxScrollToBottom = () => {
    if (messageBox.current)
      messageBox.current.scrollTo(0, messageBox.current.scrollHeight);
  };

  // 请求结果
  const getData = async (message: string) => {
    // 加入消息
    let id = generateId();
    dispatch({
      type: "addMessage",
      payload: {
        id: id,
        message: message.trim(),
        from_user: "me",
        to_user: "chatgpt",
      },
    });

    // 等待中
    id = generateId();
    dispatch({
      type: "addMessage",
      payload: {
        id: id,
        message: "等待中...",
        from_user: "chatgpt",
        to_user: "me",
      },
    });

    // 请求chatgpt
    let res = await APIGetChatMsg({ message });

    // chatgpt回答
    dispatch({
      type: "modifyMessage",
      payload: {
        id: id,
        message: res.data,
        from_user: "chatgpt",
        to_user: "me",
      },
    });

    messageBoxScrollToBottom();
  };

  const keys = [...msgList.keys()];
  // 上一条消息ID
  let preMessageId = useRef(null).current;

  // 弹出实例
  const popupInstance = useRef<any>();

  // 输入框实例
  const textareaInput = useRef<any>();

  return (
    <div className={styles.ljn_container}>
      <LJNPopup ref={popupInstance}>
        <LJNButton
          className="ljn_clearmessage"
          onClick={() => {
            dispatch({
              type: "clearMessage",
              payload: null,
            });
          }}
        >
          清空聊天记录
        </LJNButton>
      </LJNPopup>

      {/* 标题 */}
      <div className="ljn_header">
        <div className="ljn_header_left"></div>
        <div className="ljn_header_mid">CHATGPT</div>
        <div className="ljn_header_right">
          <span
            onClick={() => {
              popupInstance.current?.showPopup();
            }}
          >
            设置
          </span>
        </div>
      </div>

      {/* 信息窗口 */}
      <div className="ljn_message_box" ref={messageBox}>
        {msgList.size <= 0 ? (
          <LJNEmptyLine text="暂无消息" />
        ) : (
          keys.map((key, index) => {
            const item: IMessage = msgList.get(key) as IMessage;
            if (item.type === "message") {
              // 如果上一条消息与本条消息，超过1分钟则显示时间
              let preItemTime = 0;
              if (preMessageId) preItemTime = msgList.get(preMessageId).time;
              preMessageId = key;

              return (
                <div key={key}>
                  {item.time - preItemTime >= 60 * 1000 ? (
                    <LJNEmptyLine
                      key={key}
                      text={dayjs(item.time).format("YYYY-MM-DD HH:mm:ss")}
                    />
                  ) : null}

                  <div
                    className={[
                      "ljn_chat_item",
                      item.to_user === "me" ? null : "ljn_chat_item_me",
                    ].join(" ")}
                  >
                    <div className="ljn_avatar_box">
                      <img
                        className="ljn_avatar"
                        src={item.to_user === "me" ? chatgptAvatar : meAvatar}
                        alt=""
                      />
                    </div>
                    <div className="ljn_message">
                      <pre className="ljn_message_pre">
                        {item.message.replace(/\n+/, "")}
                      </pre>
                    </div>
                  </div>
                </div>
              );
            }
          })
        )}
      </div>
      {/* 输入框 */}
      <div className="ljn_inputarea">
        <div className="ljn_input">
          <textarea
            className="ljn_input_inner"
            value={message}
            ref={textareaInput}
            onTouchStart={(e) => {
              messageBoxScrollToBottom();
            }}
            onChange={(e) => {
              textareaInput.current.style.height = "43px";
              textareaInput.current.style.height = e.target.scrollHeight + "px";
              setMessage(e.target.value);
            }}
          />
        </div>
        <div className="ljn_button">
          <LJNButton
            className="ljn_button_inner"
            onClick={() => {
              if (message.trim() === "") {
                alert("请输入内容");
                return;
              }

              // 设置输入框为空
              setMessage("");

              // 请求数据
              getData(message);
            }}
          >
            发送
          </LJNButton>
        </div>
      </div>
    </div>
  );
}

// 消息列表
interface IMessage {
  id: string;
  type: "message";
  message: string;
  from_user: string;
  to_user: string;
  time: number;
}

// 时间消息
interface ITimeMessage {
  id: string;
  type: "time";
  time: number;
}
